@@ -169,8 +169,8 @@ keystone.token_info
     well as basic information about the project and user.
 
 keystone.token_auth
-    A keystoneclient auth plugin that may be used with a
-    :py:class:`keystoneclient.session.Session`. This plugin will load the
+    A keystoneauth1 auth plugin that may be used with a
+    :py:class:`keystoneauth1.session.Session`. This plugin will load the
     authentication data provided to auth_token middleware.
 
 
@@ -206,16 +206,18 @@ object is stored.
 
 """
 
+import binascii
 import datetime
 import logging
 
-from keystoneclient import access
-from keystoneclient import adapter
-from keystoneclient import auth
+from keystoneauth1 import access
+from keystoneauth1 import adapter
+from keystoneauth1 import discover
+from keystoneauth1 import exceptions as ksa_exceptions
+from keystoneauth1 import loading
+from keystoneauth1.loading import session as session_loading
 from keystoneclient.common import cms
-from keystoneclient import discover
-from keystoneclient import exceptions
-from keystoneclient import session
+from keystoneclient import exceptions as ksc_exceptions
 from oslo_config import cfg
 from oslo_serialization import jsonutils
 import pkg_resources
@@ -225,7 +227,7 @@ import webob.dec
 from keystonemiddleware.auth_token import _auth
 from keystonemiddleware.auth_token import _base
 from keystonemiddleware.auth_token import _cache
-from keystonemiddleware.auth_token import _exceptions as exc
+from keystonemiddleware.auth_token import _exceptions as ksm_exceptions
 from keystonemiddleware.auth_token import _identity
 from keystonemiddleware.auth_token import _request
 from keystonemiddleware.auth_token import _revocations
@@ -299,14 +301,15 @@ _OPTS = [
                ' high number of revocation events combined with a low cache'
                ' duration may significantly reduce performance.'),
     cfg.StrOpt('memcache_security_strategy',
-               default=None,
+               default='None',
+               choices=('None', 'MAC', 'ENCRYPT'),
+               ignore_case=True,
                help='(Optional) If defined, indicate whether token data'
                ' should be authenticated or authenticated and encrypted.'
-               ' Acceptable values are MAC or ENCRYPT.  If MAC, token data is'
-               ' authenticated (with HMAC) in the cache. If ENCRYPT, token'
-               ' data is encrypted and authenticated in the cache. If the'
-               ' value is not one of these options or empty, auth_token will'
-               ' raise an exception on initialization.'),
+               ' If MAC, token data is authenticated (with HMAC) in the cache.'
+               ' If ENCRYPT, token data is encrypted and authenticated in the'
+               ' cache. If the value is not one of these options or empty,'
+               ' auth_token will raise an exception on initialization.'),
     cfg.StrOpt('memcache_secret_key',
                default=None,
                secret=True,
@@ -397,7 +400,7 @@ def _conf_values_type_convert(conf):
         return {}
 
     opt_types = {}
-    for o in (_OPTS + _auth.AuthTokenPlugin.get_options()):
+    for o in _OPTS + _auth.OPTS:
         type_dest = (getattr(o, 'type', str), o.dest)
         opt_types[o.dest] = type_dest
         # Also add the deprecated name with the same type and dest.
@@ -415,7 +418,7 @@ def _conf_values_type_convert(conf):
             # This option is not known to auth_token.
             pass
         except ValueError as e:
-            raise exc.ConfigurationError(
+            raise ksm_exceptions.ConfigurationError(
                 _('Unable to convert the value of %(key)s option into correct '
                   'type: %(ex)s') % {'key': k, 'ex': e})
         opts[dest] = v
@@ -426,7 +429,17 @@ def _get_project_version(project):
     return pkg_resources.get_distribution(project).version
 
 
-class _BaseAuthProtocol(object):
+def _uncompress_pkiz(token):
+    # TypeError If the signed_text is not zlib compressed binascii.Error if
+    # signed_text has incorrect base64 padding (py34)
+
+    try:
+        return cms.pkiz_uncompress(token)
+    except (TypeError, binascii.Error):
+        raise ksm_exceptions.InvalidToken(token)
+
+
+class BaseAuthProtocol(object):
     """A base class for AuthProtocol token checking implementations.
 
     :param Callable app: The next application to call after middleware.
@@ -470,8 +483,6 @@ class _BaseAuthProtocol(object):
         :type request: _request.AuthTokenRequest
 
         """
-        request.remove_auth_headers()
-
         user_auth_ref = None
         serv_auth_ref = None
 
@@ -481,12 +492,12 @@ class _BaseAuthProtocol(object):
                 data, user_auth_ref = self._do_fetch_token(request.user_token)
                 self._validate_token(user_auth_ref)
                 self._confirm_token_bind(user_auth_ref, request)
-            except exc.InvalidToken:
+            except ksm_exceptions.InvalidToken:
                 self.log.info(_LI('Invalid user token'))
                 request.user_token_valid = False
             else:
                 request.user_token_valid = True
-                request.environ['keystone.token_info'] = data
+                request.token_info = data
 
         if request.service_token:
             self.log.debug('Authenticating service token')
@@ -494,38 +505,38 @@ class _BaseAuthProtocol(object):
                 _, serv_auth_ref = self._do_fetch_token(request.service_token)
                 self._validate_token(serv_auth_ref)
                 self._confirm_token_bind(serv_auth_ref, request)
-            except exc.InvalidToken:
+            except ksm_exceptions.InvalidToken:
                 self.log.info(_LI('Invalid service token'))
                 request.service_token_valid = False
             else:
                 request.service_token_valid = True
 
-        p = _user_plugin.UserAuthPlugin(user_auth_ref, serv_auth_ref)
-        request.environ['keystone.token_auth'] = p
+        request.token_auth = _user_plugin.UserAuthPlugin(user_auth_ref,
+                                                         serv_auth_ref)
 
     def _validate_token(self, auth_ref):
         """Perform the validation steps on the token.
 
         :param auth_ref: The token data
-        :type auth_ref: keystoneclient.access.AccessInfo
+        :type auth_ref: keystoneauth1.access.AccessInfo
 
         :raises exc.InvalidToken: if token is rejected
         """
-        # 0 seconds of validity means is it valid right now.
+        # 0 seconds of validity means it is invalid right now
         if auth_ref.will_expire_soon(stale_duration=0):
-            raise exc.InvalidToken(_('Token authorization failed'))
+            raise ksm_exceptions.InvalidToken(_('Token authorization failed'))
 
     def _do_fetch_token(self, token):
         """Helper method to fetch a token and convert it into an AccessInfo"""
-        data = self._fetch_token(token)
+        data = self.fetch_token(token)
 
         try:
-            return data, access.AccessInfo.factory(body=data, auth_token=token)
+            return data, access.create(body=data, auth_token=token)
         except Exception:
             self.log.warning(_LW('Invalid token contents.'), exc_info=True)
-            raise exc.InvalidToken(_('Token authorization failed'))
+            raise ksm_exceptions.InvalidToken(_('Token authorization failed'))
 
-    def _fetch_token(self, token):
+    def fetch_token(self, token):
         """Fetch the token data based on the value in the header.
 
         Retrieve the data associated with the token value that was in the
@@ -556,27 +567,17 @@ class _BaseAuthProtocol(object):
         if msg is False:
             msg = _('Token authorization failed')
 
-        raise exc.InvalidToken(msg)
+        raise ksm_exceptions.InvalidToken(msg)
 
     def _confirm_token_bind(self, auth_ref, req):
         if self._enforce_token_bind == _BIND_MODE.DISABLED:
             return
 
-        try:
-            if auth_ref.version == 'v2.0':
-                bind = auth_ref['token']['bind']
-            elif auth_ref.version == 'v3':
-                bind = auth_ref['bind']
-            else:
-                self._invalid_user_token()
-        except KeyError:
-            bind = {}
-
         # permissive and strict modes don't require there to be a bind
         permissive = self._enforce_token_bind in (_BIND_MODE.PERMISSIVE,
                                                   _BIND_MODE.STRICT)
 
-        if not bind:
+        if not auth_ref.bind:
             if permissive:
                 # no bind provided and none required
                 return
@@ -590,12 +591,12 @@ class _BaseAuthProtocol(object):
         else:
             name = self._enforce_token_bind
 
-        if name and name not in bind:
+        if name and name not in auth_ref.bind:
             self.log.info(_LI('Named bind mode %s not in bind information'),
                           name)
             self._invalid_user_token()
 
-        for bind_type, identifier in six.iteritems(bind):
+        for bind_type, identifier in six.iteritems(auth_ref.bind):
             if bind_type == _BIND_MODE.KERBEROS:
                 if req.auth_type != 'negotiate':
                     self.log.info(_LI('Kerberos credentials required and '
@@ -623,7 +624,7 @@ class _BaseAuthProtocol(object):
                 self._invalid_user_token()
 
 
-class AuthProtocol(_BaseAuthProtocol):
+class AuthProtocol(BaseAuthProtocol):
     """Middleware that handles authenticating client calls."""
 
     _SIGNING_CERT_FILE_NAME = 'signing_cert.pem'
@@ -638,29 +639,33 @@ class AuthProtocol(_BaseAuthProtocol):
         # conf value into correct type.
         self._conf = _conf_values_type_convert(conf)
 
-        # NOTE(sileht): If we don't want to use oslo.config global object
-        # we can set the paste "oslo_config_project" and the middleware
-        # will load the configuration with a local oslo.config object.
-        self._local_oslo_config = None
-        if 'oslo_config_project' in conf:
+        # NOTE(sileht, cdent): If we don't want to use oslo.config global
+        # object there are two options: set "oslo_config_project" in
+        # paste.ini and the middleware will load the configuration with a
+        # local oslo.config object or the caller which instantiates
+        # AuthProtocol can pass in an existing oslo.config as the
+        # value of the "oslo_config_config" key in conf. If both are
+        # set "olso_config_config" is used.
+        self._local_oslo_config = conf.get('oslo_config_config')
+        if (not self._local_oslo_config) and ('oslo_config_project' in conf):
             if 'oslo_config_file' in conf:
                 default_config_files = [conf['oslo_config_file']]
             else:
                 default_config_files = None
-
-            # For unit tests, support passing in a ConfigOpts in
-            # oslo_config_config.
-            self._local_oslo_config = conf.get('oslo_config_config',
-                                               cfg.ConfigOpts())
+            self._local_oslo_config = cfg.ConfigOpts()
             self._local_oslo_config(
-                {}, project=conf['oslo_config_project'],
+                [], project=conf['oslo_config_project'],
                 default_config_files=default_config_files,
                 validate_default_values=True)
 
-            self._local_oslo_config.register_opts(
-                _OPTS, group=_base.AUTHTOKEN_GROUP)
-            auth.register_conf_options(self._local_oslo_config,
-                                       group=_base.AUTHTOKEN_GROUP)
+        if self._local_oslo_config:
+            self._local_oslo_config.register_opts(_OPTS,
+                                                  group=_base.AUTHTOKEN_GROUP)
+            self._local_oslo_config.register_opts(_auth.OPTS,
+                                                  group=_base.AUTHTOKEN_GROUP)
+
+            loading.register_auth_conf_options(self._local_oslo_config,
+                                               group=_base.AUTHTOKEN_GROUP)
 
         super(AuthProtocol, self).__init__(
             app,
@@ -721,6 +726,7 @@ class AuthProtocol(_BaseAuthProtocol):
         request for use by applications. If not authenticated the request will
         be rejected or marked unauthenticated depending on configuration.
         """
+        request.remove_auth_headers()
         self._token_cache.initialize(request.environ)
 
         resp = super(AuthProtocol, self).process_request(request)
@@ -744,11 +750,16 @@ class AuthProtocol(_BaseAuthProtocol):
                 self.log.info(_LI('Deferring reject downstream'))
             else:
                 self.log.info(_LI('Rejecting request'))
-                self._reject_request()
+                raise webob.exc.HTTPUnauthorized(
+                    body='Authentication required',
+                    headers=self._reject_auth_headers)
 
         if request.user_token_valid:
-            request.set_user_headers(request.token_auth._user_auth_ref,
-                                     self._include_service_catalog)
+            user_auth_ref = request.token_auth._user_auth_ref
+            request.set_user_headers(user_auth_ref)
+
+            if self._include_service_catalog:
+                request.set_service_catalog_headers(user_auth_ref)
 
         if request.service_token and request.service_token_valid:
             request.set_service_headers(request.token_auth._serv_auth_ref)
@@ -774,17 +785,6 @@ class AuthProtocol(_BaseAuthProtocol):
         header_val = 'Keystone uri=\'%s\'' % self._auth_uri
         return [('WWW-Authenticate', header_val)]
 
-    def _reject_request(self):
-        """Redirect client to auth server.
-
-        :param env: wsgi request environment
-        :param start_response: wsgi response callback
-        :returns: HTTPUnauthorized http response
-
-        """
-        raise webob.exc.HTTPUnauthorized(body='Authentication required',
-                                         headers=self._reject_auth_headers)
-
     def _token_hashes(self, token):
         """Generate a list of hashes that the current token may be cached as.
 
@@ -820,7 +820,7 @@ class AuthProtocol(_BaseAuthProtocol):
             if cached:
                 return cached
 
-    def _fetch_token(self, token):
+    def fetch_token(self, token):
         """Retrieve a token from either a PKI bundle or the identity server.
 
         :param str token: token id
@@ -838,9 +838,8 @@ class AuthProtocol(_BaseAuthProtocol):
                 data = cached
 
                 if self._check_revocations_for_cached:
-                    # A token stored in Memcached might have been revoked
-                    # regardless of initial mechanism used to validate it,
-                    # and needs to be checked.
+                    # A token might have been revoked, regardless of initial
+                    # mechanism used to validate it, and needs to be checked.
                     self._revocations.check(token_hashes)
             else:
                 data = self._validate_offline(token, token_hashes)
@@ -849,39 +848,55 @@ class AuthProtocol(_BaseAuthProtocol):
 
                 self._token_cache.store(token_hashes[0], data)
 
-        except (exceptions.ConnectionRefused, exceptions.RequestTimeout):
-            self.log.debug('Token validation failure.', exc_info=True)
-            self.log.warning(_LW('Authorization failed for token'))
-            raise exc.InvalidToken(_('Token authorization failed'))
-        except exc.ServiceError as e:
-            self.log.critical(_LC('Unable to obtain admin token: %s'), e)
+        except (ksa_exceptions.ConnectFailure,
+                ksa_exceptions.RequestTimeout,
+                ksm_exceptions.RevocationListError,
+                ksm_exceptions.ServiceError) as e:
+            self.log.critical(_LC('Unable to validate token: %s'), e)
             raise webob.exc.HTTPServiceUnavailable()
-        except Exception:
+        except ksm_exceptions.InvalidToken:
             self.log.debug('Token validation failure.', exc_info=True)
             if token_hashes:
                 self._token_cache.store_invalid(token_hashes[0])
             self.log.warning(_LW('Authorization failed for token'))
-            raise exc.InvalidToken(_('Token authorization failed'))
+            raise
 
         return data
 
     def _validate_offline(self, token, token_hashes):
+        if cms.is_pkiz(token):
+            token_data = _uncompress_pkiz(token)
+            inform = cms.PKIZ_CMS_FORM
+        elif cms.is_asn1_token(token):
+            token_data = cms.token_to_cms(token)
+            inform = cms.PKI_ASN1_FORM
+        else:
+            # Can't do offline validation for this type of token.
+            return
+
         try:
-            if cms.is_pkiz(token):
-                verified = self._verify_pkiz_token(token, token_hashes)
-            elif cms.is_asn1_token(token):
-                verified = self._verify_signed_token(token, token_hashes)
-            else:
-                # Can't do offline validation for this type of token.
-                return
-        except exceptions.CertificateConfigError:
+            self._revocations.check(token_hashes)
+            verified = self._cms_verify(token_data, inform)
+        except ksc_exceptions.CertificateConfigError:
             self.log.warning(_LW('Fetch certificate config failed, '
                                  'fallback to online validation.'))
-        except exc.RevocationListError:
+        except ksm_exceptions.RevocationListError:
             self.log.warning(_LW('Fetch revocation list failed, '
                                  'fallback to online validation.'))
         else:
             data = jsonutils.loads(verified)
+
+            audit_ids = None
+            if 'access' in data:
+                # It's a v2 token.
+                audit_ids = data['access']['token'].get('audit_ids')
+            else:
+                # It's a v3 token
+                audit_ids = data['token'].get('audit_ids')
+
+            if audit_ids:
+                self._revocations.check_by_audit_id(audit_ids)
+
             return data
 
     def _validate_token(self, auth_ref):
@@ -889,7 +904,7 @@ class AuthProtocol(_BaseAuthProtocol):
 
         if auth_ref.version == 'v2.0' and not auth_ref.project_id:
             msg = _('Unable to determine service tenancy.')
-            raise exc.InvalidToken(msg)
+            raise ksm_exceptions.InvalidToken(msg)
 
     def _cms_verify(self, data, inform=cms.PKI_ASN1_FORM):
         """Verifies the signature of the provided data's IAW CMS syntax.
@@ -906,13 +921,15 @@ class AuthProtocol(_BaseAuthProtocol):
                 return cms.cms_verify(data, signing_cert_path,
                                       signing_ca_path,
                                       inform=inform).decode('utf-8')
-            except cms.subprocess.CalledProcessError as err:
+            except (ksc_exceptions.CMSError,
+                    cms.subprocess.CalledProcessError) as err:
                 self.log.warning(_LW('Verify error: %s'), err)
-                raise
+                msg = _('Token authorization failed')
+                raise ksm_exceptions.InvalidToken(msg)
 
         try:
             return verify()
-        except exceptions.CertificateConfigError:
+        except ksc_exceptions.CertificateConfigError:
             # the certs might be missing; unconditionally fetch to avoid racing
             self._fetch_signing_cert()
             self._fetch_ca_cert()
@@ -920,29 +937,12 @@ class AuthProtocol(_BaseAuthProtocol):
             try:
                 # retry with certs in place
                 return verify()
-            except exceptions.CertificateConfigError as err:
+            except ksc_exceptions.CertificateConfigError as err:
                 # if this is still occurring, something else is wrong and we
                 # need err.output to identify the problem
                 self.log.error(_LE('CMS Verify output: %s'), err.output)
                 raise
 
-    def _verify_signed_token(self, signed_text, token_ids):
-        """Check that the token is unrevoked and has a valid signature."""
-        self._revocations.check(token_ids)
-        formatted = cms.token_to_cms(signed_text)
-        verified = self._cms_verify(formatted)
-        return verified
-
-    def _verify_pkiz_token(self, signed_text, token_ids):
-        self._revocations.check(token_ids)
-        try:
-            uncompressed = cms.pkiz_uncompress(signed_text)
-            verified = self._cms_verify(uncompressed, inform=cms.PKIZ_CMS_FORM)
-            return verified
-        # TypeError If the signed_text is not zlib compressed
-        except TypeError:
-            raise exc.InvalidToken(signed_text)
-
     def _fetch_signing_cert(self):
         self._signing_directory.write_file(
             self._SIGNING_CERT_FILE_NAME,
@@ -954,41 +954,45 @@ class AuthProtocol(_BaseAuthProtocol):
             self._identity_server.fetch_ca_cert())
 
     def _get_auth_plugin(self):
-        # NOTE(jamielennox): Ideally this would use get_from_conf_options
+        # NOTE(jamielennox): Ideally this would use load_from_conf_options
         # however that is not possible because we have to support the override
-        # pattern we use in _conf_get. There is a somewhat replacement for this
-        # in keystoneclient in load_from_options_getter which should be used
-        # when available. Until then this is essentially a copy and paste of
-        # the ksc load_from_conf_options code because we need to get a fix out
-        # for this quickly.
-
-        # FIXME(jamielennox): update to use load_from_options_getter when
-        # https://review.openstack.org/162529 merges.
-
-        # !!! - UNDER NO CIRCUMSTANCES COPY ANY OF THIS CODE - !!!
+        # pattern we use in _conf_get. This function therefore does a manual
+        # version of load_from_conf_options with the fallback plugin inline.
 
         group = self._conf_get('auth_section') or _base.AUTHTOKEN_GROUP
-        plugin_name = self._conf_get('auth_plugin', group=group)
-        plugin_kwargs = dict()
 
-        if plugin_name:
-            plugin_class = auth.get_plugin_class(plugin_name)
-        else:
-            plugin_class = _auth.AuthTokenPlugin
-            # logger object is a required parameter of the default plugin
-            plugin_kwargs['log'] = self.log
+        # NOTE(jamielennox): auth_plugin was deprecated to auth_type. _conf_get
+        # doesn't handle that deprecation in the case of conf dict options so
+        # we have to manually check the value
+        plugin_name = (self._conf_get('auth_type', group=group)
+                       or self._conf.get('auth_plugin'))
+
+        if not plugin_name:
+            return _auth.AuthTokenPlugin(
+                log=self.log,
+                auth_admin_prefix=self._conf_get('auth_admin_prefix',
+                                                 group=group),
+                auth_host=self._conf_get('auth_host', group=group),
+                auth_port=self._conf_get('auth_port', group=group),
+                auth_protocol=self._conf_get('auth_protocol', group=group),
+                identity_uri=self._conf_get('identity_uri', group=group),
+                admin_token=self._conf_get('admin_token', group=group),
+                admin_user=self._conf_get('admin_user', group=group),
+                admin_password=self._conf_get('admin_password', group=group),
+                admin_tenant_name=self._conf_get('admin_tenant_name',
+                                                 group=group)
+            )
+
+        # Plugin option registration is normally done as part of the load_from
+        # function rather than the register function so copy here.
+        plugin_loader = loading.get_plugin_loader(plugin_name)
+        plugin_opts = loading.get_auth_plugin_conf_options(plugin_loader)
 
-        plugin_opts = plugin_class.get_options()
         (self._local_oslo_config or CONF).register_opts(plugin_opts,
                                                         group=group)
 
-        for opt in plugin_opts:
-            val = self._conf_get(opt.dest, group=group)
-            if val is not None:
-                val = opt.type(val)
-            plugin_kwargs[opt.dest] = val
-
-        return plugin_class.load_from_options(**plugin_kwargs)
+        getter = lambda opt: self._conf_get(opt.dest, group=group)
+        return plugin_loader.load_from_options_getter(getter)
 
     def _determine_project(self):
         """Determine a project name from all available config sources.
@@ -1034,14 +1038,14 @@ class AuthProtocol(_BaseAuthProtocol):
         # same as calling Session.load_from_conf_options(CONF, GROUP)
         # however we can't do that because we have to use _conf_get to
         # support the paste.ini options.
-        sess = session.Session.construct(dict(
+        sess = session_loading.Session().load_from_options(
             cert=self._conf_get('certfile'),
             key=self._conf_get('keyfile'),
             cacert=self._conf_get('cafile'),
             insecure=self._conf_get('insecure'),
             timeout=self._conf_get('http_connect_timeout'),
             user_agent=self._build_useragent_string()
-        ))
+        )
 
         auth_plugin = self._get_auth_plugin()
 
@@ -1070,18 +1074,14 @@ class AuthProtocol(_BaseAuthProtocol):
             env_cache_name=self._conf_get('cache'),
             memcached_servers=self._conf_get('memcached_servers'),
             use_advanced_pool=self._conf_get('memcache_use_advanced_pool'),
-            memcache_pool_dead_retry=self._conf_get(
-                'memcache_pool_dead_retry'),
-            memcache_pool_maxsize=self._conf_get('memcache_pool_maxsize'),
-            memcache_pool_unused_timeout=self._conf_get(
-                'memcache_pool_unused_timeout'),
-            memcache_pool_conn_get_timeout=self._conf_get(
-                'memcache_pool_conn_get_timeout'),
-            memcache_pool_socket_timeout=self._conf_get(
-                'memcache_pool_socket_timeout'),
+            dead_retry=self._conf_get('memcache_pool_dead_retry'),
+            maxsize=self._conf_get('memcache_pool_maxsize'),
+            unused_timeout=self._conf_get('memcache_pool_unused_timeout'),
+            conn_get_timeout=self._conf_get('memcache_pool_conn_get_timeout'),
+            socket_timeout=self._conf_get('memcache_pool_socket_timeout'),
         )
 
-        if security_strategy:
+        if security_strategy.lower() != 'none':
             secret_key = self._conf_get('memcache_secret_key')
             return _cache.SecureTokenCache(self.log,
                                            security_strategy,
@@ -1108,7 +1108,7 @@ def app_factory(global_conf, **local_conf):
 
 
 # NOTE(jamielennox): Maintained here for public API compatibility.
-InvalidToken = exc.InvalidToken
-ServiceError = exc.ServiceError
-ConfigurationError = exc.ConfigurationError
-RevocationListError = exc.RevocationListError
+InvalidToken = ksm_exceptions.InvalidToken
+ServiceError = ksm_exceptions.ServiceError
+ConfigurationError = ksm_exceptions.ConfigurationError
+RevocationListError = ksm_exceptions.RevocationListError
